@@ -1,10 +1,16 @@
+import hashlib
+import hmac
+from http import HTTPStatus
 import json
 import logging
+import math
 import re
+import time
 from typing import Any, Awaitable, Callable, Dict, List, Optional, Text
 
 from rasa.core.channels.channel import InputChannel, OutputChannel, UserMessage
-from rasa.utils.common import raise_warning
+from rasa.shared.constants import DOCS_URL_CONNECTORS_SLACK
+import rasa.shared.utils.io
 from sanic import Blueprint, response
 from sanic.request import Request
 from sanic.response import HTTPResponse
@@ -20,23 +26,39 @@ class SlackBot(OutputChannel):
     def name(cls) -> Text:
         return "slack"
 
-    def __init__(self, token: Text, slack_channel: Optional[Text] = None) -> None:
+    def __init__(
+        self,
+        token: Text,
+        slack_channel: Optional[Text] = None,
+        thread_id: Optional[Text] = None,
+        proxy: Optional[Text] = None,
+    ) -> None:
 
         self.slack_channel = slack_channel
-        self.client = WebClient(token, run_async=True)
+        self.thread_id = thread_id
+        self.proxy = proxy
+        self.client = WebClient(token, run_async=True, proxy=proxy)
         super().__init__()
 
     @staticmethod
     def _get_text_from_slack_buttons(buttons: List[Dict]) -> Text:
         return "".join([b.get("title", "") for b in buttons])
 
+    async def _post_message(self, channel, **kwargs: Any):
+        if self.thread_id:
+            await self.client.chat_postMessage(
+                channel=channel, **kwargs, thread_ts=self.thread_id
+            )
+        else:
+            await self.client.chat_postMessage(channel=channel, **kwargs)
+
     async def send_text_message(
         self, recipient_id: Text, text: Text, **kwargs: Any
     ) -> None:
         recipient = self.slack_channel or recipient_id
         for message_part in text.strip().split("\n\n"):
-            await self.client.chat_postMessage(
-                channel=recipient, as_user=True, text=message_part, type="mrkdwn",
+            await self._post_message(
+                channel=recipient, as_user=True, text=message_part, type="mrkdwn"
             )
 
     async def send_image_url(
@@ -44,16 +66,17 @@ class SlackBot(OutputChannel):
     ) -> None:
         recipient = self.slack_channel or recipient_id
         image_block = {"type": "image", "image_url": image, "alt_text": image}
-        await self.client.chat_postMessage(
-            channel=recipient, as_user=True, text=image, blocks=[image_block],
+
+        await self._post_message(
+            channel=recipient, as_user=True, text=image, blocks=[image_block]
         )
 
     async def send_attachment(
         self, recipient_id: Text, attachment: Dict[Text, Any], **kwargs: Any
     ) -> None:
         recipient = self.slack_channel or recipient_id
-        await self.client.chat_postMessage(
-            channel=recipient, as_user=True, attachments=[attachment], **kwargs,
+        await self._post_message(
+            channel=recipient, as_user=True, attachments=[attachment], **kwargs
         )
 
     async def send_text_with_buttons(
@@ -68,7 +91,7 @@ class SlackBot(OutputChannel):
         text_block = {"type": "section", "text": {"type": "plain_text", "text": text}}
 
         if len(buttons) > 5:
-            raise_warning(
+            rasa.shared.utils.io.raise_warning(
                 "Slack API currently allows only up to 5 buttons. "
                 "Since you added more than 5, slack will ignore all of them."
             )
@@ -83,7 +106,8 @@ class SlackBot(OutputChannel):
                     "value": button["payload"],
                 }
             )
-        await self.client.chat_postMessage(
+
+        await self._post_message(
             channel=recipient,
             as_user=True,
             text=text,
@@ -93,9 +117,9 @@ class SlackBot(OutputChannel):
     async def send_custom_json(
         self, recipient_id: Text, json_message: Dict[Text, Any], **kwargs: Any
     ) -> None:
-        json_message.setdefault("channel", self.slack_channel or recipient_id)
+        channel = json_message.get("channel", self.slack_channel or recipient_id)
         json_message.setdefault("as_user", True)
-        await self.client.chat_postMessage(**json_message)
+        await self._post_message(channel=channel, **json_message)
 
 
 class SlackInput(InputChannel):
@@ -114,9 +138,12 @@ class SlackInput(InputChannel):
         return cls(
             credentials.get("slack_token"),
             credentials.get("slack_channel"),
+            credentials.get("proxy"),
             credentials.get("slack_retry_reason_header", "x-slack-retry-reason"),
             credentials.get("slack_retry_number_header", "x-slack-retry-num"),
             credentials.get("errors_ignore_retry", None),
+            credentials.get("use_threads", False),
+            credentials.get("slack_signing_secret", None),
         )
         # pytype: enable=attribute-error
 
@@ -124,9 +151,12 @@ class SlackInput(InputChannel):
         self,
         slack_token: Text,
         slack_channel: Optional[Text] = None,
+        proxy: Optional[Text] = None,
         slack_retry_reason_header: Optional[Text] = None,
         slack_retry_number_header: Optional[Text] = None,
         errors_ignore_retry: Optional[List[Text]] = None,
+        use_threads: Optional[bool] = False,
+        slack_signing_secret: Optional[Text] = None,
     ) -> None:
         """Create a Slack input channel.
 
@@ -143,19 +173,45 @@ class SlackInput(InputChannel):
                 the bot posts, or channel name (e.g. '#bot-test')
                 If not set, messages will be sent back
                 to the "App" DM channel of your bot's name.
-            slack_retry_reason_header: Slack HTTP header name indicating reason that slack send retry request.
-            slack_retry_number_header: Slack HTTP header name indicating the attempt number
+            proxy: A Proxy Server to route your traffic through
+            slack_retry_reason_header: Slack HTTP header name indicating reason
+                that slack send retry request.
+            slack_retry_number_header: Slack HTTP header name indicating
+                the attempt number.
             errors_ignore_retry: Any error codes given by Slack
                 included in this list will be ignored.
                 Error codes are listed
                 `here <https://api.slack.com/events-api#errors>`_.
+            use_threads: If set to `True`, your bot will send responses in Slack as
+                a threaded message. Responses will appear as a normal Slack message
+                if set to `False`.
+            slack_signing_secret: Slack creates a unique string for your app and
+                shares it with you. This allows us to verify requests from Slack
+                with confidence by verifying signatures using your signing secret.
 
         """
         self.slack_token = slack_token
         self.slack_channel = slack_channel
+        self.proxy = proxy
         self.errors_ignore_retry = errors_ignore_retry or ("http_timeout",)
         self.retry_reason_header = slack_retry_reason_header
         self.retry_num_header = slack_retry_number_header
+        self.use_threads = use_threads
+        self.slack_signing_secret = slack_signing_secret
+
+        self._raise_deprecation_warnings()
+
+    def _raise_deprecation_warnings(self) -> None:
+        """Raises any deprecation warning regarding configuration parameters."""
+        if self.slack_signing_secret is None:
+            rasa.shared.utils.io.raise_deprecation_warning(
+                "Your slack bot is missing a configured signing secret. Running a "
+                "bot without a signing secret is deprecated and will be removed in "
+                "the next release (2.1.0). You should add a `slack_signing_secret` "
+                "parameter to your channel configuration.",
+                warn_until_version="2.1.0",
+                docs=DOCS_URL_CONNECTORS_SLACK,
+            )
 
     @staticmethod
     def _is_app_mention(slack_event: Dict) -> bool:
@@ -172,9 +228,9 @@ class SlackInput(InputChannel):
             return False
 
     @staticmethod
-    def _is_user_message(slack_event: Dict) -> bool:
+    def _is_user_message(slack_event: Dict[Text, Any]) -> bool:
         return (
-            slack_event.get("event")
+            slack_event.get("event") is not None
             and (
                 slack_event.get("event", {}).get("type") == "message"
                 or slack_event.get("event", {}).get("type") == "app_mention"
@@ -184,7 +240,9 @@ class SlackInput(InputChannel):
         )
 
     @staticmethod
-    def _sanitize_user_message(text, uids_to_remove) -> Text:
+    def _sanitize_user_message(
+        text: Text, uids_to_remove: Optional[List[Text]]
+    ) -> Text:
         """Remove superfluous/wrong/problematic tokens from a message.
 
         Probably a good starting point for pre-formatting of user-provided text
@@ -200,6 +258,8 @@ class SlackInput(InputChannel):
             str: parsed and cleaned version of the input text
         """
 
+        uids_to_remove = uids_to_remove or []
+
         for uid_to_remove in uids_to_remove:
             # heuristic to format majority cases OK
             # can be adjusted to taste later if needed,
@@ -211,9 +271,10 @@ class SlackInput(InputChannel):
             ]:
                 text = re.sub(regex, replacement, text)
 
-        """Find multiple mailto or http links like <mailto:xyz@rasa.com|xyz@rasa.com> or '<http://url.com|url.com>in text and substitute it with original content
-        """
-
+        # Find multiple mailto or http links like
+        # <mailto:xyz@rasa.com|xyz@rasa.com> or
+        # <http://url.com|url.com> in text and substitute
+        # it with original content
         pattern = r"(\<(?:mailto|http|https):\/\/.*?\|.*?\>)"
         match = re.findall(pattern, text)
 
@@ -293,17 +354,24 @@ class SlackInput(InputChannel):
                 f" due to {retry_reason}."
             )
 
-            return response.text(None, status=201, headers={"X-Slack-No-Retry": 1})
+            return response.text(
+                None, status=HTTPStatus.CREATED, headers={"X-Slack-No-Retry": 1}
+            )
 
         if metadata is not None:
             output_channel = metadata.get("out_channel")
+            if self.use_threads:
+                thread_id = metadata.get("thread_id")
+            else:
+                thread_id = None
         else:
             output_channel = None
+            thread_id = None
 
         try:
             user_msg = UserMessage(
                 text,
-                self.get_output_channel(output_channel),
+                self.get_output_channel(output_channel, thread_id),
                 sender_id,
                 input_channel=self.name(),
                 metadata=metadata,
@@ -326,13 +394,79 @@ class SlackInput(InputChannel):
             Metadata extracted from the sent event payload. This includes the output channel for the response,
             and users that have installed the bot.
         """
-        slack_event = request.json
-        event = slack_event.get("event", {})
+        content_type = request.headers.get("content-type")
 
-        return {
-            "out_channel": event.get("channel"),
-            "users": slack_event.get("authed_users"),
-        }
+        # Slack API sends either a JSON-encoded or a URL-encoded body depending on the content
+        if content_type == "application/json":
+            # if JSON-encoded message is received
+            slack_event = request.json
+            event = slack_event.get("event", {})
+            thread_id = event.get("thread_ts", event.get("ts"))
+
+            return {
+                "out_channel": event.get("channel"),
+                "thread_id": thread_id,
+                "users": slack_event.get("authed_users"),
+            }
+
+        if content_type == "application/x-www-form-urlencoded":
+            # if URL-encoded message is received
+            output = request.form
+            payload = json.loads(output["payload"][0])
+            message = payload.get("message", {})
+            thread_id = message.get("thread_ts", message.get("ts"))
+            return {
+                "out_channel": payload.get("channel", {}).get("id"),
+                "thread_id": thread_id,
+                "users": payload.get("user", {}).get("id"),
+            }
+
+        return {}
+
+    def is_request_from_slack_authentic(self, request: Request) -> bool:
+        """Validate a request from Slack for its authenticity.
+
+        Checks if the signature matches the one we expect from Slack. Ensures
+        we don't process request from a third-party disguising as slack.
+
+        Args:
+            request: incoming request to be checked
+
+        Returns:
+            `True` if the request came from Slack.
+        """
+        if self.slack_signing_secret is None:
+            # this is done for backwards compatibility, but will be removed in
+            # the next release
+            return True
+
+        try:
+            slack_signing_secret = bytes(self.slack_signing_secret, "utf-8")
+
+            slack_signature = request.headers.get("X-Slack-Signature", "")
+            slack_request_timestamp = request.headers.get(
+                "X-Slack-Request-Timestamp", "0"
+            )
+
+            if abs(time.time() - int(slack_request_timestamp)) > 60 * 5:
+                # The request timestamp is more than five minutes from local time.
+                # It could be a replay attack, so let's ignore it.
+                return False
+
+            prefix = f"v0:{slack_request_timestamp}:".encode("utf-8")
+            basestring = prefix + request.body
+            digest = hmac.new(
+                slack_signing_secret, basestring, hashlib.sha256
+            ).hexdigest()
+            computed_signature = f"v0={digest}"
+
+            return hmac.compare_digest(computed_signature, slack_signature)
+        except Exception as e:
+            logger.error(
+                f"Failed to validate slack request authenticity. "
+                f"Assuming invalid request. Error: {e}"
+            )
+            return False
 
     def blueprint(
         self, on_new_message: Callable[[UserMessage], Awaitable[Any]]
@@ -345,7 +479,51 @@ class SlackInput(InputChannel):
 
         @slack_webhook.route("/webhook", methods=["GET", "POST"])
         async def webhook(request: Request) -> HTTPResponse:
-            if request.form:
+            content_type = request.headers.get("content-type")
+
+            if not self.is_request_from_slack_authentic(request):
+                return response.text(
+                    "Message is not properly signed with a valid "
+                    "X-Slack-Signature header",
+                    status=HTTPStatus.BAD_REQUEST,
+                )
+
+            # Slack API sends either a JSON-encoded or a URL-encoded body
+            # depending on the content
+
+            if content_type == "application/json":
+                # if JSON-encoded message is received
+                output = request.json
+                event = output.get("event", {})
+                user_message = event.get("text", "")
+                sender_id = event.get("user", "")
+                metadata = self.get_metadata(request)
+
+                if "challenge" in output:
+                    return response.json(output.get("challenge"))
+
+                if not self._is_user_message(output):
+                    logger.debug(
+                        "Received message from Slack which doesn't look like "
+                        "a user message. Skipping message."
+                    )
+                    return response.text("Bot message delivered.")
+
+                if not self._is_supported_channel(output, metadata):
+                    logger.warning(
+                        f"Received message on unsupported channel: {metadata['out_channel']}"
+                    )
+                    return response.text("channel not supported.")
+
+                return await self.process_message(
+                    request,
+                    on_new_message,
+                    text=self._sanitize_user_message(user_message, metadata["users"]),
+                    sender_id=sender_id,
+                    metadata=metadata,
+                )
+            elif content_type == "application/x-www-form-urlencoded":
+                # if URL-encoded message is received
                 output = request.form
                 payload = json.loads(output["payload"][0])
 
@@ -357,40 +535,14 @@ class SlackInput(InputChannel):
                         return await self.process_message(
                             request, on_new_message, text, sender_id, metadata
                         )
-                    elif payload["actions"][0]["type"] == "button":
+                    if payload["actions"][0]["type"] == "button":
                         # link buttons don't have "value", don't send their clicks to bot
                         return response.text("User clicked link button")
                 return response.text(
-                    "The input message could not be processed.", status=500
+                    "The input message could not be processed.",
+                    status=HTTPStatus.INTERNAL_SERVER_ERROR,
                 )
 
-            elif request.json:
-                output = request.json
-                event = output.get("event", {})
-                user_message = event.get("text", "")
-                sender_id = event.get("user", "")
-                metadata = self.get_metadata(request)
-
-                if "challenge" in output:
-                    return response.json(output.get("challenge"))
-
-                elif self._is_user_message(output) and self._is_supported_channel(
-                    output, metadata
-                ):
-                    return await self.process_message(
-                        request,
-                        on_new_message,
-                        text=self._sanitize_user_message(
-                            user_message, metadata["users"]
-                        ),
-                        sender_id=sender_id,
-                        metadata=metadata,
-                    )
-                else:
-                    logger.warning(
-                        f"Received message on unsupported channel: {metadata['out_channel']}"
-                    )
-
             return response.text("Bot message delivered.")
 
         return slack_webhook
@@ -402,9 +554,11 @@ class SlackInput(InputChannel):
             or metadata["out_channel"] == self.slack_channel
         )
 
-    def get_output_channel(self, channel: Optional[Text] = None) -> OutputChannel:
+    def get_output_channel(
+        self, channel: Optional[Text] = None, thread_id: Optional[Text] = None
+    ) -> OutputChannel:
         channel = channel or self.slack_channel
-        return SlackBot(self.slack_token, channel)
+        return SlackBot(self.slack_token, channel, thread_id, self.proxy)
 
     def set_output_channel(self, channel: Text) -> None:
         self.slack_channel = channel
