@@ -1,10 +1,10 @@
 import typing
-from typing import Text, List, Any, Type
+from typing import Text, List, Any, Type, Optional
 
 from rasa.nlu.tokenizers.tokenizer import Token, Tokenizer
 from rasa.nlu.components import Component
 from rasa.nlu.utils.spacy_utils import SpacyNLP
-from rasa.nlu.training_data import Message
+from rasa.shared.nlu.training_data.message import Message
 
 from rasa.nlu.constants import SPACY_DOCS
 
@@ -25,21 +25,28 @@ class SpacyTokenizer(Tokenizer):
         "intent_tokenization_flag": False,
         # Symbol on which intent should be split
         "intent_split_symbol": "_",
+        # Regular expression to detect tokens
+        "token_pattern": None,
     }
 
-    def get_doc(self, message: Message, attribute: Text) -> "Doc":
+    def get_doc(self, message: Message, attribute: Text) -> Optional["Doc"]:
         return message.get(SPACY_DOCS[attribute])
 
     def tokenize(self, message: Message, attribute: Text) -> List[Token]:
         doc = self.get_doc(message, attribute)
+        if not doc:
+            return []
 
-        return [
+        tokens = [
             Token(
                 t.text, t.idx, lemma=t.lemma_, data={POS_TAG_KEY: self._tag_of_token(t)}
             )
             for t in doc
+            if t.text and t.text.strip()
         ]
 
+        return self._apply_token_pattern(tokens)
+
     @staticmethod
     def _tag_of_token(token: Any) -> Text:
         import spacy
